@@ -9,7 +9,7 @@ use Schedule::Activity::Message;
 use Schedule::Activity::Node;
 use Schedule::Activity::NodeFilter;
 
-our $VERSION='0.2.2';
+our $VERSION='0.2.3';
 
 sub new {
 	my ($ref,%opt)=@_;
@@ -58,8 +58,8 @@ sub _validateConfig {
 		if(!is_hashref($config{messages})) { push @errors,'Messages invalid structure' }
 		else {
 		while(my ($namea,$msga)=each %{$config{messages}}) {
-			if(!is_hashref($msga)) { push @errors,"Messages $namea invalid structure" }
-			elsif(defined($$msga{attributes})&&!is_hashref($$msga{attributes})) { push @errors,"Messages $namea invalid attributes" }
+			if(!is_hashref($msga)) { push @errors,"Messages $namea invalid structure"; next }
+			elsif(defined($$msga{attributes})&&!is_hashref($$msga{attributes})) { push @errors,"Messages $namea invalid attributes"; delete($$msga{attributes}) }
 			else { foreach my $kv (Schedule::Activity::Message::attributesFromConf($msga)) { push @errors,$attr->register($$kv[0],%{$$kv[1]}) } }
 			if(is_hashref($$msga{message})) {
 				if(defined($$msga{message}{alternates})&&!is_arrayref($$msga{message}{alternates})) { push @errors,"Messages $namea invalid alternates" }
@@ -313,15 +313,14 @@ sub scheduler {
 		else      { $res[$i][3]=-$dt }
 	}
 	#
-	# Full materialization of messages and attributes occurs after all
-	# slack/buffer adjustments have been made.  Node attributes are
-	# the 'defaults', which message attributes applying later.  This
-	# means that all attributes will be applied, but 'set' operations in
-	# messages will 'win'.
+	# Message selection occurs in _nodeMessage during path construction.
+	# Message attributes apply during path construction to permit node
+	# filtering, but final materialization occurs after slack/buffer
+	# adjustments have been made.
 	#
-	# In the future, message selection may occur during path construction,
-	# to achieve goals of node filtering, but random message selection
-	# here will only see the single message in that result.
+	# Both nodes and their messages may change attributes, but node
+	# attributes are applied first, so message attributes will "win" if
+	# both contain 'set' operations.  Documented in "/Precedence".
 	#
 	foreach my $i (0..$#res) {
 		my $node=$res[$i][1];
@@ -456,11 +455,11 @@ __END__
 
 =head1 NAME
 
-Schedule::Activity - Generate random activity schedules
+Schedule::Activity - Generate activity schedules
 
 =head1 VERSION
 
-Version 0.2.2
+Version 0.2.3
 
 =head1 SYNOPSIS
 
@@ -477,7 +476,7 @@ Version 0.2.2
         'action 1'=>{
           message=>'Begin action 1',
           tmmin=>5,tmavg=>10,tmmax=>15,
-          next=>['action 2'],
+          next=>['action 1','action 2'],
         },
         'action 2'=>{
           message=>'Begin action 2',
@@ -503,45 +502,52 @@ Version 0.2.2
 
 =head1 DESCRIPTION
 
-This module permits building schedules of I<activities> each containing randomly-generated lists of I<actions>.  This two-level approach uses explicit I<goal> times to construct the specified list of activities.  Within activities, actions are chosen within configured limits, possibly with randomization and cycling, using I<slack> and I<buffer> timing adjustments to achieve the goal.
+This module permits building schedules of I<activities> each containing randomly-generated lists of I<actions>.  Each activity is scheduled to a target time by selecting randomly selecting actions within the configured graph, which may contain repetition and cycles, and by using I<slack> and I<buffer> timing adjustments.  Attributes may be attached to events and messages, both for reporting and to control scheduling toward a I<goal>.  Annotations permit construction of secondary messages around events.
 
-For additional examples, see the C<samples/> directory.
-
-Areas subject to change are documented below.  Configurations and goals may lead to cases that currently C<die()>, so callers should plan to trap and handle these exceptions accordingly.
+For additional examples, see the tutorial and the C<samples/> directory.
 
 =head1 CONFIGURATION
 
+=head2 Overview
+
 A configuration for scheduling contains the following sections:
 
   %configuration=(
     node       =>{...}
-    attributes =>...  # see below
-    annotations=>...  # see below
-    messages   =>...  # see below
+    attributes =>{...} # optional
+    annotations=>{...} # optional
+    messages   =>{...} # optional
   )
 
+The named C<node> entries specify the activities and actions used during schedule construction.  The activity/action keys are used to configure the relationship between nodes, but the I<message> configuration within each node is the primary value used when formatting scheduling results.
+
+All other sections are optional and described below.
+
+=head2 Activities and Actions
+
 Both activities and actions are configured as named C<node> entries.  With this structure, an action and activity may have the same C<message>, but must use different key names.
 
   'activity name'=>{
-    message=>...    # an optional message string or object
-    next   =>[...], # list of child node names
-    finish =>'activity conclusion',
-    #
-    (time specification)
-    (attributes specification)
+    tmavg     =>value, ...,
+    next      =>[...],
+    finish    =>'activity conclusion',
+    message   =>...    # optional
+    attributes=>{...}, # optional
   }
   'action name'=>{
-    message=>...    # an optional message string or object
-    next   =>[...], # list of child node names
-    #
-    (time specification)
-    (attributes specification)
+    tmavg     =>value, ...,
+    next      =>[...],
+    message   =>...    # optional
+    attributes=>{...}, # optional
+    require   =>{...}, # optional
   }
 
 The list of C<next> nodes is a list of names, which must be defined in the configuration.  During schedule construction, entries will be I<chosen randomly> from the list of C<next> nodes.  The conclusion must be reachable from the initial activity, or scheduling will fail.  There is no further restriction on the items in C<next>:  Scheduling specifically supports cyclic/recursive actions, including self-cycles.
 
 There is no functional difference between activities and actions except that a node must contain C<finish> to be used for activity scheduling.  Nomenclature is primarily to support schedule organization:  A collection of random actions is used to build an activity; a sequence of activities is used to build a schedule.
 
+The C<require> configuration specifies attribute value prerequisites, computed from the current attribute values, that must be met for an action node to be a candidate for random selection.  See L<Schedule::Activity::NodeFilter> for available filtering criteria, but see L</"SCHEDULING ALGORITHM"> regarding attribute value consistency.
+
 =head2 Time specification
 
 The only time specification currently supported is:
@@ -555,29 +561,26 @@ Values must be non-negative numbers.  All three values may be identical.  Note t
 
 The slack is the amount of time that could be reduced in an action before it would need to be removed/replaced in the schedule.  The buffer is the amount of time that could be added to an action before additional actions would be needed in the schedule.
 
+Caution:  While startup/conclusion of activities may have fixed time specifications (including zero time), it is recommended, though not mandatory, that actions always contain some slack/buffer.  There is currently no "relaxing mechanism" during scheduling, so a configuration with no slack nor buffer must exactly meet the goal time requested to succeed.
+
 Providing any time value will automatically set any missing values at the fixed ratios 3,4,5.  EG, specifying only C<tmmax=40> will set C<tmmin=24> and C<tmavg=32>.  If provided two time values, priority is given to C<tmavg> to set the third.
 
 Scheduling may be controlled with the tension settings described below.  Future changes may support automatic slack/buffering, univeral slack/buffer ratios, and open-ended/relaxed slack/buffering.
 
 =head2 Messages
 
-Each activity/action node may contain an optional message.  Messages are provided so the caller can easily format the returned schedules.  While message attributes may be used during schedule, the message strings themselves are not used during scheduling.  Messages may be:
+See L<Schedule::Activity::Message/CONFIGURATION> for all possible message configurations.
 
-  message=>'A message string'
-  message=>'named message key'
-  message=>['An array','of alternates','chosen randomly']
-  message=>{name=>'named message key'}
-  message=>{
-    alternates=>[
-      {message=>'A hash containing an array', attributes=>{...}}
-      {message=>'of alternates',              attributes=>{...}}
-      {message=>'with optional attributes',   attributes=>{...}}
-      {message=>'named message key'}
-      {name=>'named message key'}
-    ]
-  }
+Any activity or action may contain an optional message string or configuration.  Messages permit the caller to easily format generated schedules.  Messages may contain attributes, which can affect subsequent scheduling.  Message attributes are emitted with the attribute response values.
 
-Message selection is randomized for arrays and a hash of alternates.  Named messages must exist (see L</"NAMED MESSAGES"> below).  Any attributes are emitted with the attribute response values, described below.
+A scheduling configuration may declare a list of named messages, which will be available to all message configurations:
+
+  %configuration=(
+    messages=>{
+      'key name'=>{ message configuration }
+      ...
+    },
+  )
 
 =head1 RESPONSE
 
@@ -586,90 +589,55 @@ The response from C<schedule(activities=>[...])> is:
   %schedule=(
     error=>['list of validation errors, if any',...],
     activities=>[
-      [seconds, message],
+      [seconds, event],
       ..,
     ],
-    annotations=>{
-      'group'=>{
-        events=>[
-          [seconds, message],
-        ]
-      },
-      ...
-    },
+    annotations=>{...}
     attributes=>{
-      name=>{
-        y  =>(final value),
-        xy =>[[tm,value],...],
-        avg=>(average, depends on type),
-      },
+      name=>{attribute report},
       ...
     },
   )
 
-=head2 Failures
+=head2 Success Response
 
-In addition to validation failures returned through C<error>, the following may cause the scheduler to C<die()>:  The activity name is undefined.  The scheduler was not able to reach the named finish node.  The number of retries or backtracking attempts has been exhausted.
+When scheduling is successful, the list of events is in the C<activities> array.  Each event is an array containing the timestamp, and an event node containing a C<{message}>.  Scheduling always occurs in order, so activities should appear in non-decreasing timestamp order.  Timestamp units are undefined, so formatting is the responsibility of the caller.
+
+The C<event{message}> is materialized during schedule construction, so the response will only contain the single, chosen message for the event.  Alternate messages attached to the underlying event node, or a referenced name message, are not included in the response.
+
+The "annotations" response is described below in L</ANNOTATIONS>.
+
+For the "attribute report", see L<Schedule::Activity::Attribute/RESPONSE>.
+
+=head2 Validation Errors
+
+If the configuration fails prechecks, findings will be reported in the C<error> array and scheduling will not be attempted.  Configuration errors include invalid type/structures, invalid node keys/values, messages/attributes/annotations, as well as basic activity/action reachability checks.  Note that node filtering (via a "require" configuration) may prevent scheduling despite these basic reachability checks.
+
+=head2 Unhandled Errors
+
+In addition to validation failures returned through C<error>, the following may cause the scheduler to C<die()>:  The requested activity name is undefined.  The scheduler was not able to reach the named finish node.  The number of retries or backtracking attempts has been exhausted.
 
 The difference between the result time and the goal may cause retries when an excess exceeds the available slack, or when a shortage exceeds the available buffer.
 
-Caution:  While startup/conclusion of activities may have fixed time specifications, at this time it is recommended that actions always contain some slack/buffer.  There is currently no "relaxing mechanism" during scheduling, so a configured with no slack nor buffer must exactly meet the goal time requested.
-
-=head1 SCHEDULING ALGORITHM
-
-The configuration of the C<next> actions is the primary contributor to the schedules that can be built.  As with all algorithms of this type, there are many configurations that simply won't work well:  For example, this is not a maze solver, a best path finder, nor a resourcing optimization system.  Scheduling success toward the stated goals generally requires that actions have different C<tmmin>, C<tmmax>, and C<tmavg>, and that actions permit reasonable repetition and recursion.  Highly imbalanced actions, such as a branch of length 10 and another of length 5000, may always fail depending on the goal.  Neverthless, for the activities and actions so described, how does it work?
-
-The scheduler is a randomized, opportunistic, single-step path growth algorithm.  An activity starts at the indicated node.  At each step, the C<next> entries are filtered and a random action is chosen, then the process repeats.  The selection of the next step is restricted based on the I<current time> (at the end of the action) as follows.
-
-First, a I<random current time> is computed based on the current time, the accumulated slack and buffer, and the tension settings (see below).  If the random current time is less than the goal time, the next action will be a random non-final node, if available, or the final node if all other choices are filtered or unavailable.
-
-If the random current time is greater than the goal time and the final action is listed as a C<next> action, it will be chosen.
-
-In all other cases, a random C<next> action will be chosen.
-
-=head2 Tension
-
-Schedule construction proceeds toward the goal time incrementally, with each action appending its C<tmavg> until the goal is reached.  If the accumulated average times were exactly equal to the goal for the activity, schedules would be unambiguous.  For repeating, recursive scheduling, however, it's necessary to consider scenarios where the actions don't quite reach the goal or where they extend beyond the goal.
-
-=head3 Buffer and Slack
-
-Each activity node and action has buffer and slack, as defined above, that contributes to the accumulated total buffer and slack.  The amount of buffer/slack that contributes to the random current time is controlled by including C<schedule(tensionbuffer=E<gt>value)> and C<tensionslack=E<gt>value>, each between 0 and 1.  Tension effectively controls how little of each contributes toward randomization around the goal.
-
-In the 'laziest' mode, with C<tension=0.0>, all available buffer/slack is used to establish the random current time, increasing the likelihood that it is greater than the goal.  With a lower buffer tension, for example, scheduling is more likely to reach the final activity node sooner, and thus will contain a smaller number of actions on average, each stretched toward C<tmmax>.  With a higher tension, the goal time must be met (or exceeded) before aggressively seeking the final activity node, so schedules will contain a larger number of actions, each compressed toward C<tmmin>.
-
-The tension for slack is similar, with lower values permitting a larger number of actions beyond the goal, each compressed toward C<tmmin>, whereas with tension near 1, scheduling will seek the final activity node as soon as the schedule time exceeds the goal, resulting in a smaller number of activities.
-
-The random computed time is a uniform distribution around the current time, but because actions are scheduled incrementally, this leads to a skewed distribution that favors a smaller number of actions.  See C<samples/tension.png> for the distributions where exactly 100 repeated actions would be expected.
-
-The default values are 0.5 for the slack tension, and ~0.85 for the buffer tension.  This gives an expected number of actions that is very close to C<goal/tmavg>, roughly plus 10% minus 5%.
-
-=head3 Response
-
-The scheduling response contains C<{stat}> that reports the accumulated slack and buffer used for all actions, as well as C<slackttl> and C<bufferttl> which represent the maximum available.  The amount of slack used during scheduling is C<slack/slackttl>, and the same for buffer.  These values can assist with choosing tension settings based on the specific configuration.
-
 =head1 ATTRIBUTES
 
-Attributes permit tracking boolean or numeric values during schedule construction.  The result of C<schedule> contains attribute information that can be used to verify or adjust the schedule.
-
-=head2 Types
-
-The two types of attributes are C<bool> or C<int>, which is the default.  A boolean attribute is primarily used as a state flag.  An integer attribute can be used both as a counter or gauge, either to track the number of occurrences of an activity or event, or to log varying numeric values.
+Attributes permit tracking boolean or numeric values that can be used to affect node selection during schedule construction.  The resulting attribute history can be used to verify the final schedule or compute goal scores.
 
 =head2 Configuration
 
-Multiple attributes can be referenced from any activity/action.  For example:
+For a complete description of attribute configuration options, see L<Schedule::Activity::Attribute>.
 
-  'activity/action name'=>{
+Attributes may be referenced from an activity, action, or message.  For example:
+
+  'action name'=>{
     attributes=>{
       temperature=>{set=>value, incr=>value, decr=>value, note=>'comment'},
-      counter    =>{set=>value, incr=>value, note=>'comment'},
-      flag       =>{set=>0/1, note=>'comment'},
+      counter    =>{set=>value, incr=>value},
+      flag       =>{set=>0/1},
     },
   }
 
-Any attribute may include a C<note> for convenience, but this value is not stored nor reported.
-
-The main configuration can also declare attribute names and starting values.  It is recommended to set any non-zero initial values in this fashion, since calling C<set> requires that activity to always be the first requested in the schedule.  Boolean values must be declared in this section:
+The scheduling configuration may also declare attribute names and starting values.
 
   %configuration=(
     attributes=>{
@@ -679,41 +647,10 @@ The main configuration can also declare attribute names and starting values.  It
     },
   )
 
+Boolean types must be declared in this section.  It is recommended to set any non-zero initial values in this fashion, since calling C<set> requires that activity to always be the first requested in the schedule.  
+
 Attributes within message alternate configurations and named messages are identified during configuration validation.  Together with activity/action configurations, attributes are verified before schedule construction, which will fail if an attribute name is referenced in a conflicting manner.
 
-=head2 Response values
-
-The response from C<schedule> includes an C<attributes> section as:
-
-  attributes=>{
-    name=>{
-      y  =>(final value),
-      xy =>[[tm,value],...],
-      avg=>(average, depends on type),
-    },
-    ...
-  }
-
-The C<y> value is the last recorded value in the schedule.  The C<xy> contains an array of all values and the times at which they changed; see Logging.  The C<avg> is roughly the time-weighted average of the value, but this depends on the attribute type.
-
-If an activity containing a unique attribute is not used during construction, the attribute will still be included in the response with its default and initial value.
-
-=head2 Integer attributes
-
-The C<int> type is the default for attributes.  If initialized in C<%configuration>, it may specify the type, or the value, or both.  The default value is zero, but this may be overwritten if the first activity node specifically calls C<set>.
-
-Integer attributes within activity/actions support all of:  C<set>, C<incr>, C<decr>.  There is no actual restriction on type so any Perl L<number> is valid, integers or real numbers, positive or negative.
-
-The reported C<avg> is the overall time-weighted average of the values, computed via a trapezoid rule.  That is, if C<tm=0, value=2> and C<tm=10, value=12>, the average is 7 with a weight of 10.  See Logging for more details.
-
-=head2 Boolean attributes
-
-The C<bool> type must be declared in C<%configuration>.  The value may be specified, but defaults to zero/false.
-
-Boolean attributes within activity/actions support:  C<set>.  Currently there is no restriction on values, but the behavior is only defined for values 0/1.
-
-The reported C<avg> is the percentage of time in the schedule for which the flag was true.  That is, if C<tm=0, value=0>, and C<tm=7, value=1>, and C<tm=10, value=1> is the complete schedule, then the reported average for the boolean will be C<0.3>.
-
 =head2 Precedence
 
 When an activity/action node and a selected message both contain attributes, the value of the attribute is updated first from the action node and then from the message node.  For boolean attributes, this means the "value set in the message has precedence".  For integer attributes, suppose that the value is initially zero; then, if both the action and message have attribute operators, the result will be:
@@ -724,13 +661,9 @@ When an activity/action node and a selected message both contain attributes, the
   set=5   incr=6    11
   incr=7  incr=8    15
 
-=head2 Logging
+=head2 Average Values
 
-The reported C<xy> is an array of values of the form C<(tm, value)>, with each representing an activity/action referencing that attribute built into the schedule.  Each attribute will have its initial value of C<(0, value)>, either the default or the value specified in C<configuration{attributes}>.
-
-Any attribute may be "fixed" in the log at their current value with the configuration C<name=E<gt>{}>, which is equivalent to C<incr=0> for integers.
-
-Attribute logging always occurs at the beginning and end of the completed schedule, so that all scheduled time affects the weighted average value calculation.  Activities may reset or fix attributes in their beginning or final node; the final node is only the "end of the activity" when C<tmavg=0>.
+Attributes are always logged at the beginning and end of the completed schedule, so that all scheduled time affects the weighted average value calculation.  Activities may reset or fix attributes as needed in their beginning or final node; note that the final node is only the "end of the activity" when C<tmavg=0>.
 
 =head2 Recomputation
 
@@ -738,22 +671,35 @@ Any schedule of activities associated with the initial configuration can generat
 
   %attributes=$scheduler->computeAttributes(@activities)
 
-This permits manual modification of activities, merging across multiple scheduling runs, or merging of annotations (below) to materialize a final attribute report.  This does not affect the attributes within the C<$scheduler>.
+This permits manual modification of activities, merging across multiple scheduling runs, or merging of annotations (below) to materialize a final attribute report.  This does not affect the attributes within the C<$scheduler> object itself.
 
 =head1 ANNOTATIONS
 
-A scheduling configuration may contain a list of annotations:
+=head2 Overview
+
+Annotations are secondary messages and/or attributes that are attached to the scheduling configuration and are inserted around activity/action nodes.  Annotations are divided into named I<groups>, permitting separation by category, and each named group contains a list of annotations (or "notes").
+
+=head2 Configuration
+
+Annotations are configuration in the C<annotations> section of the scheduling configuration:
 
   %configuration=(
     annotations=>{
       'annotation group'=>[
         {annotation configuration},
         ...
-      ]
+      ],
+      ...
     },
   )
 
-Scheduling I<annotations> are a collection of secondary events to be attached to the built schedule and are configured as described in L<Schedule::Activity::Annotation>.  Each named group can have one or more annotation.  Each annotation will be inserted around the matching actions in the schedule and be reported from C<schedule> in the annotations section as:
+Each named group is an array, and each note configuration is a hash, as described in L<Schedule::Activity::Annotation>.  Annotations may use named messages from the scheduling configuration.
+
+Within an individual group, earlier annotations take priority if two events are scheduled at the same time.  Because groups are generated separately, multiple groups of annotations may have conflicting event times in the results.  Note that the C<between> setting is only enforced for each annotation individually at this time, and not for notes within the same group.
+
+=head2 Response
+
+Annotation groups are generated after scheduling is complete and are reported in the annotations section as:
 
   annotations=>{
     'group'=>{
@@ -764,47 +710,80 @@ Scheduling I<annotations> are a collection of secondary events to be attached to
     },
   }
 
-Within an individual group, earlier annotations take priority if two events are scheduled at the same time.  Multiple groups of annotations may have conflicting event schedules with event overlap.  Note that the C<between> setting is only enforced for each annotation individually at this time.
+Messages in the response are materialized using any named messages, and have the same structure as the message response for activity/action events.
 
-Annotations do I<not> update the C<attributes> response from C<schedule>.  Because annotations may themselves contain attributes, they are retained separately from the main schedule of activities to permit easier rebuilding.  At this time, however, the caller must verify that annotation schedules before merging them and their attributes into the schedule.  Annotations may also be built separately after schedule construction as described in L<Schedule::Activity::Annotation>.
+Annotations do I<not> update the C<attributes> response from C<schedule>.  Because annotations may themselves contain attributes, they are retained separately from the main schedule of activities to permit easier rebuilding.
 
-Annotations may use named messages, and messages in the annotations response structure are materialized using the named message configuration passed to C<schedule>.
+=head2 Merging
 
-=head1 NAMED MESSAGES
+At this time, the caller must merge groups of annotations into C<schedule{activities}> manually.  Group order may matter, and the behavior of overlapping or nearby event times must be prioritized based on needs.  When constructing schedules incrementally, it is recommended to use the C<nonote> option described in L</"Incremental Construction">.
 
-A scheduling configuration may contain a list of common messages.  This is particularly useful when there are a large number of common alternate messages where copy/pasting through the scheduling configuration would be egregious.
+Rudimentary merging mechanisms are provided in C<schedule-activity.pl>.
 
-  %configuration=(
-    messages=>{
-      'key name'=>{ any regular message configuration }
-      ...
-    },
-  )
+=head1 SCHEDULING ALGORITHM
 
-Any message configuration within activity/action nodes may then reference the message by its key as shown above.  During message selection, any string message or configured C<name> will return the message configuration for C<key=name>, if it exists, or will return the string message.  If a configured message string matches a referenced name, the name takes precedence.
+=head2 Overview
 
-The configuration of a named message may only create string, array, or hash alternative messages; it cannot reference another name.
+The configuration of the C<next> actions is the primary contributor to the schedules that can be built.  As with all algorithms of this type, there are many configurations that simply won't work well:  For example, this is not a maze solver, a best path finder, nor a resourcing optimization system.  Scheduling success toward the stated goals generally requires that actions have different C<tmmin>, C<tmmax>, and C<tmavg>, and that actions permit reasonable repetition and recursion.  Highly imbalanced actions, such as a branch of length 10 and another of length 5000, may always fail depending on the goal.  Neverthless, for the activities and actions so described, how does it work?
 
-=head1 FILTERING
+The scheduler is a randomized, opportunistic, single-step path growth algorithm.  An activity starts at the indicated node.  At each step, the C<next> entries are filtered and a random action is chosen, then the process repeats.  The selection of the next step is restricted based on the I<current time> (at the end of the action) as follows.
 
-Action nodes may include prerequisites before they will be selected during scheduling:
+First, a I<random current time> is computed based on the current time, the accumulated slack and buffer, and the tension settings (see below).  If the random current time is less than the goal time, the next action will be a random non-final node, if available, or the final node if all other choices are filtered or unavailable.
 
-  'action name'=>{
-    require=>{
-      ...
-    }
-    ...
-  }
+If the random current time is greater than the goal time and the final action is listed as a C<next> action, it will be chosen.
 
-During schedule construction, the list of C<next> actions will be filtered by C<require> to identify candidate actions.  The current attribute values at the time of selection will be used to perform the evaluation.  The available filtering criteria are fully described in L<Schedule::Activity::NodeFilter> and include attribute numeric comparison and Boolean operators.
+In all other cases, a random C<next> action will be chosen.
 
-Action filtering may be used, together with attribute setting and increments, to prevent certain actions from appearing if others have not previously occurred, or vice versa.  This mechanism may also be used to specify global or per-activity limits on certain actions.
+=head2 Buffer and Slack
 
-=head2 Slack and Buffer
+Schedule construction proceeds toward the goal time incrementally, with each action appending its C<tmavg> until the goal is reached.  If the accumulated average times were exactly equal to the goal for the activity, schedules would be unambiguous.  For repeating, recursive scheduling, however, it's necessary to consider scenarios where the actions don't quite reach the goal or where they extend beyond the goal.
 
-During scheduling, filtering is evaluated as a I<single pass> only, per activity:  When finding a sequence of actions to fulfill a scheduling goal for an activity, candidates (from C<next>) are checked based on the current attributes.  Action times during construction are based on C<tmavg>, so any filter using attribute average values will be computed as if the action sequence only used C<tmavg>.  After a solution is found, however, actions are adjusted across the total slack/buffer available, so the "materialized average" attribute values can be slightly different.
+Each activity node and action has buffer and slack, as defined above, that contributes to the accumulated total buffer and slack.  The amount of buffer/slack that contributes to the random current time is controlled by including C<schedule(tensionbuffer=E<gt>value)> and C<tensionslack=E<gt>value>, each between 0 and 1.  Tension effectively controls how little of each contributes toward randomization around the goal.
 
-This should never affect attributes used for a stateful/flag/counter-based filter, because those value changes will still occur in the same sequence.
+In the 'laziest' mode, with C<tension=0.0>, all available buffer/slack is used to establish the random current time, increasing the likelihood that it is greater than the goal.  With a lower buffer tension, for example, scheduling is more likely to reach the final activity node sooner, and thus will contain a smaller number of actions on average, each stretched toward C<tmmax>.  With a higher tension, the goal time must be met (or exceeded) before aggressively seeking the final activity node, so schedules will contain a larger number of actions, each compressed toward C<tmmin>.
+
+The tension for slack is similar, with lower values permitting a larger number of actions beyond the goal, each compressed toward C<tmmin>, whereas with tension near 1, scheduling will seek the final activity node as soon as the schedule time exceeds the goal, resulting in a smaller number of activities.
+
+The random computed time is a uniform distribution around the current time, but because actions are scheduled incrementally, this leads to a skewed distribution that favors a smaller number of actions.  See C<samples/tension.png> for the distributions where exactly 100 repeated actions would be expected.
+
+The default values are 0.5 for the slack tension, and approximately 0.85 for the buffer tension.  This gives an expected number of actions that is very close to C<goal/tmavg> (skewed distribution as plus 10% minus 5%).
+
+The scheduling response contains C<{stat}> that reports the accumulated slack and buffer used for all actions, as well as C<slackttl> and C<bufferttl> which represent the maximum available.  The amount of slack used during scheduling is C<slack/slackttl>, and the same for buffer.  These values can assist with choosing tension settings based on the specific configuration.
+
+=head2 Consistency
+
+Attributes may be used for filtering during schedule construction.  When scheduling an activity, a temporary history of attributes is built.  Each activity/action node will update attributes, after which any materialized message will update attributes.  Node filtering applies to the last recorded attributes, independent of any actions a candidate node may take on attributes.  Filtering occurs before random node selection.
+
+After reaching the target time for an activity, event times are updated based on the total slack/buffer time available.  The actual attribute history is constructed from those adjusted times, and will be visible to the next activity scheduled.
+
+Node filtering does not currently support attribute average values, however:  Filtering is evaluated as a I<single pass> only, so average values visible during filtering may be slightly different than averages after slack/buffer adjustments.
+
+Annotations are computed separately by groups.  Attributes arising from merged annotations do not affect attributes retroactively (nor, obviously, any node filtering).  See L</Recomputation>.
+
+Scheduling proceeds stepwise, and "consistency" is defined as adherence to the computed values I<at that time>.  No recomputation occurs, except through full retries (such as goal seeking).  All of the following can create paradoxes that are avoided with this approach:  Slack/buffer adjustments can alter attribute average values leading to different node filtering/selection.  Slack/buffer adjustments can produce times that open other action branches that may have be unavailable during scheduling.  Annotations that are merged may adjust attributes such that the nodes they are annotating would disappear.  Nodes may adjust attributes such that the final node is unreachable.
+
+=head2 Incremental Construction
+
+For longer schedules with multiple activities, regenerating a full schedule because of issues with a single activity can be time consuming.  A more interactive approach would be to build and verify the first activity, then review choices for the second activity schedule, append it, and continue.  After full scheduling construction, annotations can be built.
+
+Incremental schedules can be built using the C<after> and C<nonote> options:
+
+  # Use nonote to avoid annotation build at this time
+  my $choiceA=$scheduler->schedule(nonote=>1, activities=>[[600,'activity1']);
+  my $choiceB=$scheduler->schedule(nonote=>1, activities=>[[600,'activity1']);
+  #
+  # two or more choices are reviewed and one is selected
+  my %res=$scheduler->schedule(after=>$choiceB, activities=>[[600,'activity2']);
+
+The schedule indicated via C<after> signals that the scheduler should build the activities and extend the schedule.  Attributes are automatically loaded from the earlier part of the schedule and affect node filtering normally.
+
+Annotations, which may apply to any node by name, are dropped when the schedule is extended.  This is because a single annotation may have a limit and match nodes across activities, so full regeneration is necessary.  To make generation more efficient, C<nonote> may be set to skip annotation generation in earlier steps.
+
+The final result above does generate annotations, but it's also possible to pass C<nonote> at each step and then generate annotations without adding activities by calling:
+
+  my %res=$scheduler->schedule(after=>$earlierSchedule, activities=>[]);
+
+This functionality is experimental starting with Version 0.2.1.
 
 =head1 IMPORT MECHANISMS
 
@@ -829,29 +808,6 @@ The imported configuration permits an activity to be followed by any of its acti
 
 The full settings needed to build a schedule can be loaded with C<%settings=loadMarkdown(text)>, and both C<$settings{configuration}> and C<$settings{activities}> will be defined so an immediate call to C<schedule(%settings)> can be made.
 
-=head1 INCREMENTAL CONSTRUCTION
-
-For longer schedules with multiple activities, regenerating a full schedule because of issues with a single activity can be time consuming.  A more interactive approach would be to build and verify the first activity, then review choices for the second activity schedule, append it, and continue.  After full scheduling construction, annotations can be built.
-
-Incremental schedules can be built using the C<after> and C<nonote> options:
-
-  # Use nonote to avoid annotation build at this time
-  my $choiceA=$scheduler->schedule(nonote=>1, activities=>[[600,'activity1']);
-  my $choiceB=$scheduler->schedule(nonote=>1, activities=>[[600,'activity1']);
-  #
-  # two or more choices are reviewed and one is selected
-  my %res=$scheduler->schedule(after=>$choiceB, activities=>[[600,'activity2']);
-
-The schedule indicated via C<after> signals that the scheduler should build the activities and extend the schedule.  Attributes are automatically loaded from the earlier part of the schedule and affect node filtering normally.
-
-Annotations, which may apply to any node by name, are dropped when the schedule is extended.  This is because a single annotation may have a limit and match nodes across activities, so full regeneration is necessary.  To make generation more efficient, C<nonote> may be set to skip annotation generation in earlier steps.
-
-The final result above does generate annotations, but it's also possible to pass C<nonote> at each step and then generate annotations without adding activities by calling:
-
-  my %res=$scheduler->schedule(after=>$earlierSchedule, activities=>[]);
-
-This functionality is experimental starting with Version 0.2.1.
-
 =head1 BUGS
 
 It is possible for some settings to get stuck in an infinite loop:  Be cautious setting C<tmavg=0> for actions.
